clear all
close all
clc 

load 3Coupled_Micros_Initial_Data
  
figure
hold on
for i=1:size(Sparam,1)
    for j=i:size(Sparam,2)
    plot(frequency*1e-9,20*log10(abs(squeeze(Sparam(i,j,:)))),'b');
    end
end
title('Bandpass S-parameters')
xlabel('Frequency (GHz)','fontsize',14);
axis tight
ylabel('|S| (dB)','fontsize',14);
box on
set(gca,'fontsize',14);

fc=frequency(271); %%% center frequency = 57GHz for conversion to baseband  
 
figure
hold on
for i=1:size(Sparam,1)
    for j=i:size(Sparam,2)
    plot((frequency-fc)*1e-9,20*log10(abs(squeeze(Sparam(i,j,:)))),'r');
    end
end
title('Baseband S-parameters','fontsize',14);
xlabel('Frequency (GHz)')
axis tight
ylabel('|S| (dB)','fontsize',14);
box on
set(gca,'fontsize',14);

aux=round(linspace(1,length(frequency),401));
Spar_modeling=Sparam(:,:,aux);
freq_modeling=frequency(aux)-fc;    %%% Only a subset of frequency samples is used to compute a CVF model, 
                                    %%% while its accuracy is verified on the entire frequency samples available: 
                                    %%% this procedure allows one to verify the model accuracy in the entire bandwidth considered and to check for overfitting
 
                                       
%%%%%%%%%%%%%%% Computing CVF Model                                    

%%% Options to compute the CVF model: a full description is provided in the file VFdriver_complex.m

%%% The number of initial poles and their value must be specified by the user 
initial_poles=[];               %%% Vector of initial poles. Specify "poles=[]" if automated pole generation is to be used. 
setting_fitting.N=69;           %%% Number of poles of the CVF model

%%% If no other fitting options are defined in the structure "setting_fitting", the default values are used to compute the CVF model.
%%% For example, lines 54-62 can be commented 
setting_fitting.plot=0;         %%% The CVF modeling results are not plotted
setting_fitting.screen=0;       %%% No information are shown in the Matlab command window during fitting

setting_fitting.Niter1=10;      %%% Number of iterations for fitting sum of elements
setting_fitting.Niter2=10;      %%% Number of iterations for matrix fitting
setting_fitting.asymp=2;        %%% If equal to 1 --> The CVF model has D=0,  E=0  ('Strictly proper')
                                %%% If equal to 2 --> The CVF model has D~=0, E=0  ('Proper')

setting_fitting.stable=1;       %%% If equal to 1 a stable CVF model is computed 

 
%%% Options for the passivity assessment and enforcement of the CVF model: a full description is provided in the file RPdriver_complex.m

mod_passive=1;                       %%% If equal to 1 a passive CVF model is computed 

setting_passivity=[];                %%% Use default options for passivity enforcement. 
                                     %%% Possible modifications of the default options include:
                                            %%% setting_passivity.outputlevel=0;     %%% If equal to 0,  a minimum level of output information are shown in the Matlab command window during passivity enforcement
                                            %%% setting_passivity.Niter_out=15;      %%% Maximum n.o. iterations in outer-loop of the passivity enforcement algorithm
                                            %%% setting_passivity.Niter_in=4;        %%% Maximum n.o. iterations in inner-loop of the passivity enforcement algorithm
                     

[CVF_model, Spar_CVF, err_CVF, sys_CVF]= Create_Rat_Mod_CVF(Spar_modeling,freq_modeling,initial_poles,setting_fitting,mod_passive,setting_passivity); 
 
% CVF_model: data structure with
    % the poles and residues of the CVF model, namely CVF_model.poles and CVF_model.R
    % the corresponding state-space matrices: CVF_model.A, CVF_model.B, CVF_model.C, CVF_model.D
% Spar_CVF: frequency response computed via the CVF model on the modeling frequencies 
% err_CVF: maximum absolute model error (dB) between the scattering parameters and the corresponding CVF model
% sys_CVF: the CVF model is converted into a Matlab continuous-time state-space model (ss): see Control System toolbox.
    
   
%%%%%%%%%%%%%%% Validating CVF Model Accuracy 
 
Spar_CVF_all_frequencies=freqresp(sys_CVF,2*pi*(frequency-fc));  %%% Computing the CVF model response for all frequency samples via its equivalent Matlab ss representation

err_vf_val=db(max(reshape(Sparam-Spar_CVF_all_frequencies,[],1)));
disp(['Modeling error for all frequency samples: ' num2str(err_vf_val)])

figure
hold on
for i=1:size(Sparam,1)
    for j=i:size(Sparam,2)
    plot((frequency-fc)*1e-9,db(squeeze(Sparam(i,j,:))),'r');
    hold on
    plot((frequency-fc)*1e-9,db(squeeze(Spar_CVF_all_frequencies(i,j,:))),'b--');
    plot((frequency-fc)*1e-9,db(squeeze(Sparam(i,j,:))-squeeze(Spar_CVF_all_frequencies(i,j,:))),'g-')
    end
end
title(['Complex VF with ' num2str(setting_fitting.N) ' poles'])
xlabel('Frequency (GHz)','fontsize',14);
axis tight
ylabel('|S| (dB)','fontsize',14);
box on
h_legend=legend('S-param data','CVF model','Modeling error');
set(h_legend,'FontSize',14);
set(gca,'fontsize',14);






